"""A mock DeviceAdapter that just connects to a python MockIOTileDevice
"""
from iotile.core.hw.transport.adapter import DeviceAdapter
from mock_iotile import RPCInvalidIDError, TileNotFoundError, RPCNotFoundError

class MockDeviceAdapter(DeviceAdapter):
    """A mock DeviceAdapter that connects to one or more python MockIOTileDevices
    """

    def __init__(self):
        self.devices = {}
        self.connections = {}

        super(MockDeviceAdapter, self).__init__()

    def add_device(self, conn_string, device):
        self.devices[conn_string] = device

    def can_connect(self):
        return True

    def advertise(self):
        for conn_string, device in self.devices.iteritems():
            info = {'uuid': device.iotile_id, 'connection_string': conn_string, 'signal_strength': 0}
            self._trigger_callback('on_scan', self.id, info, 0)

    def connect_async(self, connection_id, connection_string, callback):
        if connection_string not in self.devices:
            callback(connection_id, self.id, False, "Could not find device connection string")
            return

        self.connections[connection_id] = self.devices[connection_string]
        callback(connection_id, self.id, True, "")

    def disconnect_async(self, connection_id, callback):
        if connection_id not in self.connections:
            callback(connection_id, self.id, False, "Could not find connection id to disconnect")
            return

        del self.connections[connection_id]
        callback(connection_id, self.id, True, "")

    def _open_rpc_interface(self, connection_id, callback):
        if connection_id not in self.connections:
            callback(connection_id, self.id, False, "Could not find connection id to disconnect")
            return

        device = self.connections[connection_id]
        device.open_rpc_interface()
        
        callback(connection_id, self.id, True, "")

    def _open_script_interface(self, connection_id, callback):
        if connection_id not in self.connections:
            callback(connection_id, self.id, False, "Could not find connection id to disconnect")
            return

        device = self.connections[connection_id]
        device.open_script_interface()
        
        callback(connection_id, self.id, True, "")

    def _open_streaming_interface(self, connection_id, callback):
        if connection_id not in self.connections:
            callback(connection_id, self.id, False, "Could not find connection id to disconnect")
            return

        device = self.connections[connection_id]
        reports = device.open_streaming_interface()
        
        callback(connection_id, self.id, True, "")

        for report in reports:
            self._trigger_callback('on_report', connection_id, report)

    def send_rpc_async(self, connection_id, address, rpc_id, payload, timeout, callback):
        status = 0x00

        if connection_id not in self.connections:
            callback(connection_id, self.id, False, "Could not find connection id to send rpc", None, None)
            return

        device = self.connections[connection_id]

        try:
            payload = device.call_rpc(address, rpc_id, payload)
        except (RPCInvalidIDError, RPCNotFoundError):
            status = 0x03
        except TileNotFoundError:
            status = 0xFF

        if status == 0 and len(payload) > 0:
            status = 0xC0
        elif status == 0:
            status = 0x40

        callback(connection_id, self.id, True, "", status, payload)

    def send_script_async(self, connection_id, data, progress_callback, callback):
        if connection_id not in self.connections:
            callback(connection_id, self.id, False, "Could not find connection id to send script")
            return

        device = self.connections[connection_id]

        for i in xrange(0, len(data), 20):
            device.push_script_chunk(data[i:i+20])
            progress_callback(min(i+20, len(data)), len(data))

        callback(connection_id, self.id, True, "")

    def stop_sync(self):
        pass

    def periodic_callback(self):
        pass



# This file is adapted from python code released by WellDone International
# under the terms of the LGPLv3.  WellDone International's contact information is
# info@welldone.org
# http://welldone.org
#
# Modifications to this file from the original created at WellDone International 
# are copyright Arch Systems Inc.

#Caveats and possible issues
#Mac OS X
# - when using a virtualenv, readline is not properly installed into the virtualenv
#   and cannot be imported.  You need to install it using easy_install as described here
#   http://calvinx.com/tag/readline/

import re

from setuptools import setup, find_packages
import version

setup(
    name="iotile-test",
    packages=find_packages(exclude=("test",)),
    version=version.version,
    license="LGPLv3",
    install_requires=[
        "iotile-core>=3.0.0"
    ],
    description="IOTile Core Tools",
    entry_points={'iotile.virtual_device': ['simple = iotile.mock.devices.simple_virtual_device:SimpleVirtualDevice',
                                            'individual_reports = iotile.mock.devices.individual_report_test_device:IndividualReportTestDevice'],
                  'iotile.proxy': ['simple = iotile.mock.devices.simple_virtual_proxy']},
    author="Arch",
    author_email="info@arch-iot.com",
    url="https://github.com/iotile/coretools/iotilecore",
    keywords=["iotile", "arch", "embedded", "hardware"],
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules"
        ],
    long_description="""\
IOTileCore
------

A python package for interacting with IOTile based devices.  See https://www.arch-iot.com.

"""
)



"""Reference device for testing the individual report format
"""

from iotile.core.hw.virtual.virtualdevice import VirtualIOTileDevice, rpc, RPCInvalidIDError, RPCNotFoundError, TileNotFoundError
from iotile.core.exceptions import ArgumentError
from iotile.core.hw.reports.individual_format import IndividualReadingReport, IOTileReading

class IndividualReportTestDevice(VirtualIOTileDevice):
    """Mock IOTileDevice that creates a sequence of reports upon connection

    This device can be considered a reference implementation of the individual
    reading report format.

    Args:
        args (dict): Any arguments that you want to pass to create this device.
            Supported args are:
                iotile_id (int): The UUID used for this device (default: 1)
                num_readings (int): The number of readings to be generated every
                    time the streaming interface is opened. (default: 100)
                start_timestamp (int): The first timestamp that should be generated
                    for the first Reading sent.  Timestamps are sequential for each
                    reading, so x, x+1, etc.
                    Default: 0
                reading_generator (string): The method for generating readings. 
                    Options are: sequential, random (default: sequential)
                    random will generate random values between the reading_min and
                    reading_max keys (default: 0, 100)
                    sequential will sequentially generate reading values starting
                    at reading_start (default: 0)
    """

    def __init__(self, args):
        iotile_id = args.get('iotile_id', 1)

        if isinstance(iotile_id, basestring) or isinstance(iotile_id, unicode):
            iotile_id = int(iotile_id, 16)

        generator = args.get('reading_generator', 'sequential')            
        self.num_readings = args.get('num_readings', 100)

        stream_string = args.get('stream_id', '5001')
        self.stream_id = int(stream_string, 0)
        
        #Now pull in args for the generator
        if generator == 'sequential':
            self.reading_start = args.get('reading_start', 0)
            self.generator = self._generate_sequential
        elif generator == 'random':
            self.reading_min = args.get('reading_min', 0)
            self.reading_max = args.get('reading_max', 100)
            self.generator = self._generate_random
        else:
            raise ArgumentError("Unknown reading generator mechanism", reading_generator=generator, known_generators=['sequential', 'random'])

        super(IndividualReportTestDevice, self).__init__(iotile_id, 'Simple')

    @rpc(8, 0x0004, "", "H6sBBBB")
    def controller_name(self):
        """Return the name of the controller as a 6 byte string
        """

        status = (1 << 1) | (1 << 0) #Configured and running
        
        return [0xFFFF, self.name, 1, 0, 0, status]

    def _generate_sequential(self):
        reports = []
        
        for i in xrange(self.reading_start, self.num_readings+self.reading_start):
            reading = IOTileReading(i-self.reading_start, self.stream_id, i)
            report = IndividualReadingReport.FromReadings(self.iotile_id, [reading])
            reports.append(report)

        return reports

    def _generate_random(self):
        return []

    def open_streaming_interface(self):
        """Called when someone opens a streaming interface to the device

        Returns:
            list: A list of IOTileReport objects that should be sent out
                the streaming interface.
        """

        reports = self.generator()
        return reports
